require 'test/unit'
require 'logger'
require 'pry'
require File.expand_path(File.dirname(__FILE__) + '/../rails/init')

ActiveRecord::Migration.verbose = false
ActiveRecord::Base.establish_connection(:adapter => "sqlite3", :database => ":memory:")

class ActsAsMailableTest < Test::Unit::TestCase

  def setup_mailable
    require File.expand_path(File.dirname(__FILE__) + '/../lib/generators/email_template/templates/create_email_templates.rb')
    CreateEmailTemplates.up
    require File.expand_path(File.dirname(__FILE__) + '/../lib/generators/email_template/templates/create_mailable_templates.rb')
    CreateMailableTemplates.up
    load(File.expand_path(File.dirname(__FILE__) + '/../lib/generators/email_template/templates/email_template.rb'))
    load(File.expand_path(File.dirname(__FILE__) + '/../lib/generators/email_template/templates/mailable_template.rb'))
  end

  def setup_test_models
    load(File.expand_path(File.dirname(__FILE__) + '/schema.rb'))
    load(File.expand_path(File.dirname(__FILE__) + '/models.rb'))
  end

  def setup
    setup_mailable
    setup_test_models
  end

  def teardown
    ActiveRecord::Base.connection.tables.each do |table|
      ActiveRecord::Base.connection.drop_table(table)
    end
  end

  def test_create_email_template
    organisation = Organisation.create(:name => "Organisation One")
    assert_not_nil organisation.email_templates.create(:name => "First template.", :subject => "This is the first template.", body: "This is the body of the first template.").id
  end

  def test_fetch_email_templates
    organisation = Organisation.create(:name => "Organisation One")
    organisation.email_templates.create(:name => "First template.", :subject => "This is the first template.", body: "This is the body of the first template.")
    mailable = Organisation.find(1)
    assert_equal 1, mailable.email_templates.length
    assert_equal "First template.", mailable.email_templates.first.name
    assert_equal "This is the first template.", mailable.email_templates.first.subject
  end

  def test_find_email_templates_for_mailable
    organisation = Organisation.create(:name => "Organisation One")
    email_template = organisation.email_templates.create(:name => "First template.", :subject => "This is the first template.", body: "This is the body of the first template.")
    assert_equal [email_template], EmailTemplate.find_email_templates_for_mailable(organisation.class.name, email_template.id)
  end

  def test_find_mailable
    organisation = Organisation.create(:name => "Organisation One")
    email_template = organisation.email_templates.create(:name => "First template.", :subject => "This is the first template.", body: "This is the body of the first template.")
    assert_equal organisation, EmailTemplate.find_mailable(organisation.class.name, email_template.id)
  end

  def test_find_email_templates_for
    organisation = Organisation.create(:name => "Organisation One")
    email_template = organisation.email_templates.create(:name => "First template.", :subject => "This is the first template.", body: "This is the body of the first template.")
    assert_equal [email_template], Organisation.find_email_templates_for(organisation)
  end

  def test_email_templates_ordered_by_submitted
    organisation = Organisation.create(:name => "Organisation One")
    email_template = organisation.email_templates.create(:name => "First template.", :subject => "This is the first template.", body: "This is the body of the first template.")
    email_template2 = organisation.email_templates.create(:name => "Second template.", :subject => "This is the second template.")
    assert_equal [email_template2, email_template], organisation.email_templates.recent
  end

  def test_add_email_template
    organisation = Organisation.create(:name => "Organisation One")
    email_template = EmailTemplate.new(:name => "First template.", :subject => "This is the first template.", body: "This is the body of the first template.")
    organisation.email_templates << email_template
    assert_equal [email_template], organisation.email_templates
  end

  def test_create_mailable_template
    organisation = Organisation.create(:name => "Organisation One")
    user = User.create(name: "James")
    email_template = organisation.email_templates.create(:name => "First template.", :subject => "This is the first template.", body: "This is the body of the first template.")
    assert_not_nil MailableTemplate.create(email_template_id: email_template.id, author_id: user.id).id
  end

end
